#!/bin/sh
# sm_install_saphana.sh 1.4 13.03.2015i PLE
# Copyright  : (c)2014 SEP AG GmbH D83629 Weyarn, all rights reserved
# This script can be used 
# to easily install the SAP HANA Database extension
# for SEP sesam on a standard environment.

# Read pathes from Sesam installation and set logfile destination
read_ini ()
{
   SM_INI=`grep -i ^sm_ini /etc/sesam2000.ini|cut -d"=" -f2`
   if [  $SM_INI ]
   then
     echo `grep -i $1 $SM_INI|cut -d"=" -f2`
   fi
}

if [ -r /etc/sesam2000.ini ]
  then
    ini_dir=`read_ini gv_rw_ini= | sed 's/ini\//ini/'`
    bin_dir=`read_ini gv_ro_bin`
    lgc_dir=`read_ini gv_rw_lgc | sed 's/lgc\//lgc/'`  
fi
LOGFILE=$lgc_dir/sm_install_saphana.log

#
# Print some information for the customer
#
echo
echo "Welcome to the SAP HANA Database extension installation script for SEP sesam. This script assumes that the HANA instance has been set up meeting the recommendations from the SAP HANA Administration guide."
echo "Before proceeding, please make sure you created two tasks (one for data backups, one for log backups) using the backup type 'SAP HANA' on your SEP sesam server and take a note of their names."
read -p " Attention: If you made a custom installation that doesn't use /usr/sap/<SID> pathes, please press Ctrl+C now and install the module manually using the documentation on our wiki (http://wiki.sepsoftware.com).

Please press ENTER to continue or Ctrl+C to abort: "
echo 
echo 

# Ask for multi-tenancy
read -p "Are you running this script for a Multi Tenancy database? Please enter no here if your system is a Single Container database. (y/n)" multitenant
case $multitenant in
        [yY])
                echo "yes" && echo "`date`: User selected that his system is a Multi Tenancy setup. Proceed in Multi-Tenancy mode..." >> $LOGFILE
		multitenant=1;;
        [nN])
                echo "no" && echo "`date`: User selected that his system is a Single Container database. Proceed in Single DB mode..." >> $LOGFILE
		multitenant=0;;
        *)
                echo "Invalid option. Exiting..." && echo "`date`: User gave unhandled input when being asked for Multi Tenancy. Exiting..." >> $LOGFILE
                exit 1;;
esac

echo "SID pathes detected on this system:"
#
# Determine the system SID and write it to variable
#
if [ -r $LOGFILE ]
	then
		echo >> $LOGFILE
fi
echo "+++ Logging started for sm_install_saphana.sh" >> $LOGFILE
echo "`date`: Output of scan in /usr/sap directory: `echo && find /usr/sap/* -maxdepth 0 -type d`" >> $LOGFILE

prompt="Please select the SID path for the SID you want to install the module for:"
options=( $(find /usr/sap/* -maxdepth 0 -type d -print0 | xargs -0) )

PS3="$prompt "
select opt in "${options[@]}" "Quit" ; do 
    if (( REPLY == 1 + ${#options[@]} )) ; then
        echo "'Quit' option selected. Exiting..."
        echo "`date`: User selected the 'Quit' option in SID choice menu. Exiting." >> $LOGFILE
        exit

    elif (( REPLY > 0 && REPLY <= ${#options[@]} )) ; then
        echo "You have selected $opt ..."
        echo "`date`: User selected $opt as desired SID path" >> $LOGFILE
        break

    else
        echo "Invalid option. Please select one of the numbers shown above."
    fi
done    
db_sidpath=$opt
echo "`date`: Variable SID was set to `echo $db_sidpath | sed 's/\/usr\/sap\///' | sed 's/\///'`" >> $LOGFILE
SID=`echo $db_sidpath | sed 's/\/usr\/sap\///' | sed 's/\///'`

#
# Generate the parameter file from user input and write it to ini directory
#
read -p "Please enter the hostname of your SEP sesam server: " SESAM_SERVER
read -p "Please enter the hostname of the device server that shall store the backup or press ENTER to use the SEP sesam server: " SESAM_TAPESERVER
read -p "Please enter the exact name of the task you defined for data backups: " SESAM_DATAJOB
read -p "Please enter the exact name of the task you defined for log backups: " SESAM_LOGJOB
read -p "Please enter the drive number of the drive you want your backup to be written to: " SESAM_DRIVE
read -p "Please enter the media pool to be used for the backup: " SESAM_POOL
read -p "Please enter the full path where your SSB logfile should be stored or press ENTER to use the sesam lgc directory: " SSB_LOGFILE
read -p "Please enter the instance number of the DB you want to configure the backup for: " instance
read -s -p "Please enter the password for the SYSTEM database user (will not be logged or saved): " syspass
echo

#
# Check input variables for validity
#
if [ -z $SESAM_SERVER ]
	then
		echo "No Sesam server was specified. Exiting..."
		echo "`date`: No Sesam server was specified. Exiting..." >> $LOGFILE
	exit
fi

if [ -z $SESAM_TAPESERVER ]
	then
		SESAM_TAPESERVER="$SESAM_SERVER"
		echo "`date`: User didn't specify a SESAM_TAPESERVER, therefor using $SESAM_SERVER ..." >> $LOGFILE
fi

if [ -z $SESAM_DATAJOB ]
	then
		echo "No data backup task name was specified. Exiting..."
		echo "`date`: No log backup task name was specified. Exiting..." >> $LOGFILE
	exit
fi

if [ -z $SESAM_LOGJOB ]
	then
		echo "No log backup task name was specified. Exiting..."
		echo "`date`: No log backup task name was specified. Exiting..." >> $LOGFILE
	exit
fi

if [ -z $SESAM_DRIVE ]
	then
		echo "No drive number was specified. Exiting..."
		echo "`date`: No drive number was specified. Exiting..." >> $LOGFILE
	else if [[ $SESAM_DRIVE != *[[:digit:]]* ]] ; 
	then
   	echo "Drive number has to be numeric. Exiting..."; 
   	echo "`date`: The user didn't enter a numeric drive number. Exiting..." >> $LOGFILE
   exit
   fi
fi

if [ -z $SESAM_POOL ]
	then
		echo "No media pool was specified. Exiting..."
		echo "`date`: No media pool was specified. Exiting..." >> $LOGFILE
	exit
fi
	
if [ -z $SSB_LOGFILE ] 
	then 
		SSB_LOGFILE="$lgc_dir/hdbbackint_$SID.log"
		echo "`date`: User didn't specify a location for the SSB_LOGFILE, therefor using $lgc_dir/hdbbackint_$SID.log ..." >> $LOGFILE
fi

if [ -z $instance ]
	then
		echo "Instance number can't be empty. Exiting..."
		echo "`date`: User didn't specify an instance number, exiting  ..." >> $LOGFILE
	exit
fi

#
# Print collected information to logfile and STDOUT, prompt user to confirm
#
echo | tee -a $LOGFILE
echo "---Installation Summary---" | tee -a $LOGFILE
echo | tee -a $LOGFILE
echo "You have entered the following values for the parameter file:" && echo "User entered the following values for the parameter file:" >> $LOGFILE
echo "SESAM_SERVER=$SESAM_SERVER" | tee -a $LOGFILE
echo "SESAM_TAPESERVER=$SESAM_TAPESERVER" | tee -a $LOGFILE
echo "SESAM_JOB=$SESAM_DATAJOB for data backups and $SESAM_LOGJOB for log backups" | tee -a $LOGFILE
echo "SESAM_DRIVE=$SESAM_DRIVE" | tee -a $LOGFILE
echo "SESAM_POOL=$SESAM_POOL" | tee -a $LOGFILE
echo "SSB_LOGFILE=$SSB_LOGFILE" | tee -a $LOGFILE
echo | tee -a $LOGFILE
echo "The selected path to your SID directory is $db_sidpath"
echo

read -p "If these values are correct, the installation will be started. Are these values correct? (y/n)" valueswitch
case $valueswitch in 
	[yY])
		echo "yes" && echo "`date`: User confirmed that the values are correct. Proceeding..." >> $LOGFILE ;;
	[nN])
		echo "You answered no. Exiting..." && echo "`date`: User declined that the information entered was correct. Exiting..." >> $LOGFILE
		exit 1;;
	*)
		echo "Invalid option. Exiting..." && echo "`date`: User gave unhandled input when being asked for starting the installation. Exiting..." >> $LOGFILE
		exit 1;;
esac

#
# Installation procedure using collected information
#
# Create directories needed for hdbbackint binary and parameter file
echo "Creating directories for hdbbackint binary and parameter file..."
mkdir -p $db_sidpath/SYS/global/hdb/opt/hdbconfig
if [ $? -eq 0 ]
	then
		echo "`date`: Successfully created directories for hdbbackint binary and parameter file link."
	else
		echo "`date`: Error while creating directories for hdbbackint binary and parameter file link. Check the filesystem permissions and make sure you run this script as root to avoid this."
fi

# Check if parameter files already exist and if so, write their contents to logfile
if [ -r $ini_dir/backint_$SID.utl ]
	then
		echo "`date`: Found already existing data parameter file. Content:" >> $LOGFILE
		cat $ini_dir/backint_$SID.utl >> $LOGFILE
		echo >> $LOGFILE
		echo "`date`: Deleting old parameter file.."
		rm -rf $ini_dir/backint_$SID.utl
fi

if [ -r $ini_dir/backint_"$SID"_log.utl ]
	then
		echo "`date`: Found already existing log parameter file. Content:" >> $LOGFILE
		cat $ini_dir/backint_"$SID"_log.utl >> $LOGFILE
		echo >> $LOGFILE
		echo "`date`: Deleting old parameter file.."
		rm -rf $ini_dir/backint_"$SID"_log.utl
fi

# Write new parameter files
echo "# Parameter file generated by sm_install_saphana.sh" >> $ini_dir/backint_$SID.utl
echo "SESAM_SERVER=$SESAM_SERVER" >> $ini_dir/backint_$SID.utl
echo "SESAM_TAPESERVER=$SESAM_TAPESERVER" >> $ini_dir/backint_$SID.utl
echo "SESAM_JOB=$SESAM_DATAJOB" >> $ini_dir/backint_$SID.utl
echo "SESAM_DRIVE=$SESAM_DRIVE" >> $ini_dir/backint_$SID.utl
echo "SESAM_POOL=$SESAM_POOL" >> $ini_dir/backint_$SID.utl
echo "SSB_LOGFILE=$SSB_LOGFILE" >> $ini_dir/backint_$SID.utl
if [ $? -eq 0 ]
	then
		echo "`date`: Successfully created and populated data backup parameter file in $ini_dir/backint_$SID.utl" | tee -a $LOGFILE
	else
		echo "`date`: Error writing parameter file information into $ini_dir/backint_$SID.utl. Could be a permission problem." | tee -a $LOGFILE
fi

echo "# Parameter file generated by sm_install_saphana.sh" >> $ini_dir/backint_"$SID"_log.utl
echo "SESAM_SERVER=$SESAM_SERVER" >> $ini_dir/backint_"$SID"_log.utl
echo "SESAM_TAPESERVER=$SESAM_TAPESERVER" >> $ini_dir/backint_"$SID"_log.utl
echo "SESAM_JOB=$SESAM_LOGJOB" >> $ini_dir/backint_"$SID"_log.utl
echo "SESAM_DRIVE=$SESAM_DRIVE" >> $ini_dir/backint_"$SID"_log.utl
echo "SESAM_POOL=$SESAM_POOL" >> $ini_dir/backint_"$SID"_log.utl
echo "SSB_LOGFILE=$SSB_LOGFILE" >> $ini_dir/backint_"$SID"_log.utl
if [ $? -eq 0 ]
	then
		echo "`date`: Successfully created and populated log backup parameter file in $ini_dir/backint_"$SID"_log.utl" | tee -a $LOGFILE
	else
		echo "`date`: Error writing parameter file information into $ini_dir/backint_"$SID"_log.utl. Could be a permission problem." | tee -a $LOGFILE
fi

# Set the permissions for the logging dir
echo "Setting permission 777 on sesam log/lgc directory..."
chmod 777 $lgc_dir
if [ $? -eq 0 ]
	then
		echo "`date`: Setting permission 777 on $lgc_dir successful." >> $LOGFILE
	else
		echo "`date`: Error setting permission 777 on $lgc_dir - please make sure you're running the script as root." >> $LOGFILE
fi

# Create symlinks
echo "Creating symlinks for hdbbackint and the parameter file..."
ln -s $bin_dir/hdbbackint $db_sidpath/SYS/global/hdb/opt/hdbbackint
if [ $? -eq 0 ]
	then
		echo "`date`: Successfully created symlink for hdbbackint executable" | tee -a $LOGFILE
	else
		echo "`date`: Error creating symlink for hdbbackint executable. Make sure you're running the script as root." | tee -a $LOGFILE
fi
ln -s $ini_dir/backint_$SID.utl $db_sidpath/SYS/global/hdb/opt/hdbconfig
if [ $? -eq 0 ]
	then
		echo "`date`: Successfully created symlink for parameterfile" | tee -a $LOGFILE
	else
		echo "`date`: Error creating symlink for parameterfile. Make sure you're running the script as root." | tee -a $LOGFILE
fi

#
# Set parameter files in database configuration for single container db
#

if [ $multitenant -eq 0 ]
        then
            echo "Setting data backup parameter file to be used in database configuration..."
            $db_sidpath/SYS/exe/hdb/hdbsql -i $instance -n localhost -u SYSTEM -p $syspass "alter system alter configuration ('global.ini','SYSTEM') set ('backup','data_backup_parameter_file') = '$ini_dir/backint_$SID.utl' with reconfigure"
            if [ $? -eq 0 ]
                then
                    echo "`date`: Successfully set data backup parameterfile destination in database." >> $LOGFILE
                else
                    echo "`date`: Error setting data backup parameterfile destination in database. Please insert $ini_dir/backint_$SID.utl manually in the SAP HANA Studio backup configuration." >> $LOGFILE
            fi

            echo "Enabling log backup via backint using log backup parameter file in database configuration..."
            $db_sidpath/SYS/exe/hdb/hdbsql -i $instance -n localhost -u SYSTEM -p $syspass "alter system alter configuration ('global.ini','SYSTEM') set ('backup','log_backup_using_backint') = 'true' with reconfigure"
            if [ $? -eq 0 ]
                then
                    echo "`date`: Successfully enabled log backup via backint in database." >> $LOGFILE
                else
                    echo "`date`: Error enabling log backup via backing in database. Please enable it manually via SAP HANA Studio." >> $LOGFILE
            fi

            $db_sidpath/SYS/exe/hdb/hdbsql -i $instance -n localhost -u SYSTEM -p $syspass "alter system alter configuration ('global.ini','SYSTEM') set ('backup','log_backup_parameter_file') = '$ini_dir/backint_"$SID"_log.utl' with reconfigure"
            if [ $? -eq 0 ]
                then
                    echo "`date`: Successfully set log backup pararmeter file in database." >> $LOGFILE
                else
                    echo "`date`: Error enabling log backup parameterfile destination in database. Please insert $ini_dir/backint_"$SID"_log.utl manually in the SAP HANA Studio backup configuration." >> $LOGFILE
            fi
fi

#
# Set parameter files in database configuration for multi-tenant db
#

if [ $multitenant -eq 1 ]
        then
            echo "Setting data backup parameter file for SYSTEMDB to be used in database configuration..."
            $db_sidpath/SYS/exe/hdb/hdbsql -i $instance -d SYSTEMDB -n localhost -u SYSTEM -p $syspass "alter system alter configuration ('global.ini','SYSTEM') set ('backup','data_backup_parameter_file') = '$ini_dir/backint_$SID.utl' with reconfigure"
                if [ $? -eq 0 ]
                    then
                        echo "`date`: Successfully set data backup parameterfile destination in database." >> $LOGFILE
                    else
                        echo "`date`: Error setting data backup parameterfile destination in database. Please insert $ini_dir/backint_$SID.utl manually in the SAP HANA Studio backup configuration." >> $LOGFILE
                fi

            echo "Enabling log backup via backint using log backup parameter file in database configuration..."
            $db_sidpath/SYS/exe/hdb/hdbsql -i $instance -d SYSTEMDB -n localhost -u SYSTEM -p $syspass "alter system alter configuration ('global.ini','SYSTEM') set ('backup','log_backup_using_backint') = 'true' with reconfigure"
                if [ $? -eq 0 ]
                    then
                        echo "`date`: Successfully enabled log backup via backint in database." >> $LOGFILE
                    else
                        echo "`date`: Error enabling log backup via backing in database. Please enable it manually via SAP HANA Studio." >> $LOGFILE
                fi

            $db_sidpath/SYS/exe/hdb/hdbsql -i $instance -d SYSTEMDB -n localhost -u SYSTEM -p $syspass "alter system alter configuration ('global.ini','SYSTEM') set ('backup','log_backup_parameter_file') = '$ini_dir/backint_"$SID"_log.utl' with reconfigure"
                if [ $? -eq 0 ]
                    then
                        echo "`date`: Successfully set log backup pararmeter file in database." >> $LOGFILE
                    else
                        echo "`date`: Error enabling log backup parameterfile destination in database. Please insert $ini_dir/backint_"$SID"_log.utl manually in the SAP HANA Studio backup configuration." >> $LOGFILE
                fi
fi

echo "##############  FINISHED  ###################" && echo "+++ Logging finished for sm_install_saphana.sh" >> $LOGFILE
echo "Please note: If you are running a multi-node SAP HANA DB, remember to run this script on every node. Have fun making backups!"
exit 0
